@@ -38,7 +38,7 @@
 
 #include <sys/types.h>
 #include <sys/wait.h>
-#include <sys/poll.h>
+#include <poll.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <netinet/in.h>
@@ -73,6 +73,7 @@ struct option general_admopt[] = {
 	{"verbose", no_argument, 0, 'v'},
 	{"config-file", required_argument, 0, 'c'},
 	{"config-to-test", required_argument, 0, 't'},
+	{"config-to-exclude", required_argument, 0, 'E'},
 	{"drbdsetup", required_argument, 0, 's'},
 	{"drbdmeta", required_argument, 0, 'm'},
 	{"drbd-proxy-ctl", required_argument, 0, 'p'},
@@ -113,7 +114,6 @@ static int sh_lres(const struct cfg_ctx *);
 static int sh_ll_dev(const struct cfg_ctx *);
 static int sh_md_dev(const struct cfg_ctx *);
 static int sh_md_idx(const struct cfg_ctx *);
-static int sh_status(const struct cfg_ctx *);
 static int adm_drbdmeta(const struct cfg_ctx *);
 static int adm_khelper(const struct cfg_ctx *);
 static int adm_setup_and_meta(const struct cfg_ctx *);
@@ -127,6 +127,7 @@ static int adm_forget_peer(const struct cfg_ctx *);
 static int adm_peer_device(const struct cfg_ctx *);
 
 int ctx_by_name(struct cfg_ctx *ctx, const char *id, checks check);
+int was_file_already_seen(char *fn);
 
 static char *get_opt_val(struct options *, const char *, char *);
 
@@ -265,7 +266,7 @@ int adm_adjust_wp(const struct cfg_ctx *ctx)
 	.show_in_usage = 4,		\
 	.res_name_required = 1,		\
 	.backend_res_name = 1,		\
-	.iterate_volumes = 0,		\
+	.iterate_volumes = 1,		\
 	.vol_id_required = 1,		\
 	.verify_ips = 0,		\
 	.uc_dialog = 1,			\
@@ -330,8 +331,8 @@ static struct adm_cmd secondary_cmd = {"secondary", adm_drbdsetup, ACF1_RESNAME
 static struct adm_cmd invalidate_cmd = {"invalidate", adm_invalidate, ACF1_MINOR_ONLY };
 static struct adm_cmd invalidate_remote_cmd = {"invalidate-remote", adm_drbdsetup, ACF1_PEER_DEVICE .takes_long = 1};
 static struct adm_cmd outdate_cmd = {"outdate", adm_outdate, ACF1_DEFAULT};
-/*  */ struct adm_cmd resize_cmd = {"resize", adm_resize, ACF1_DEFAULT .disk_required = 1};
-static struct adm_cmd verify_cmd = {"verify", adm_drbdsetup, ACF1_PEER_DEVICE};
+/*  */ struct adm_cmd resize_cmd = {"resize", adm_resize, &resize_cmd_ctx, ACF1_DEFAULT .disk_required = 1};
+static struct adm_cmd verify_cmd = {"verify", adm_drbdsetup, &verify_cmd_ctx, ACF1_PEER_DEVICE};
 static struct adm_cmd pause_sync_cmd = {"pause-sync", adm_drbdsetup, ACF1_PEER_DEVICE};
 static struct adm_cmd resume_sync_cmd = {"resume-sync", adm_drbdsetup, ACF1_PEER_DEVICE};
 static struct adm_cmd adjust_cmd = {"adjust", adm_adjust, &adjust_ctx, ACF1_RESNAME .vol_id_optional = 1};
@@ -370,13 +371,12 @@ static struct adm_cmd sh_md_dev_cmd = {"sh-md-dev", sh_md_dev, ACF2_SHELL .disk_
 static struct adm_cmd sh_md_idx_cmd = {"sh-md-idx", sh_md_idx, ACF2_SHELL .disk_required = 1};
 static struct adm_cmd sh_ip_cmd = {"sh-ip", sh_ip, ACF2_SHELL};
 static struct adm_cmd sh_lr_of_cmd = {"sh-lr-of", sh_lres, ACF2_SHELL};
-static struct adm_cmd sh_status_cmd = {"sh-status", sh_status, ACF2_GEN_SHELL};
 
 static struct adm_cmd proxy_up_cmd = {"proxy-up", adm_proxy_up, ACF2_PROXY};
 static struct adm_cmd proxy_down_cmd = {"proxy-down", adm_proxy_down, ACF2_PROXY};
 
-/*  */ struct adm_cmd new_resource_cmd = {"new-resource", adm_resource, ACF2_SH_RESNAME};
-/*  */ struct adm_cmd new_minor_cmd = {"new-minor", adm_new_minor, ACF4_ADVANCED};
+/*  */ struct adm_cmd new_resource_cmd = {"new-resource", adm_resource, &resource_options_ctx, ACF2_SH_RESNAME};
+/*  */ struct adm_cmd new_minor_cmd = {"new-minor", adm_new_minor, &device_options_ctx, ACF4_ADVANCED};
 /*  */ struct adm_cmd del_minor_cmd = {"del-minor", adm_drbdsetup, ACF1_MINOR_ONLY .show_in_usage = 4, .disk_required = 0, };
 
 static struct adm_cmd khelper01_cmd = {"before-resync-target", adm_khelper, ACF3_RES_HANDLER};
@@ -395,7 +395,7 @@ static struct adm_cmd khelper12_cmd = {"unfence-peer", adm_khelper, ACF3_RES_HAN
 
 static struct adm_cmd suspend_io_cmd = {"suspend-io", adm_drbdsetup, ACF4_ADVANCED  .backend_res_name = 0 };
 static struct adm_cmd resume_io_cmd = {"resume-io", adm_drbdsetup, ACF4_ADVANCED  .backend_res_name = 0 };
-static struct adm_cmd set_gi_cmd = {"set-gi", adm_drbdmeta, .disk_required = 1, .need_peer = 1, ACF4_ADVANCED_NEED_VOL};
+static struct adm_cmd set_gi_cmd = {"set-gi", adm_drbdmeta, &wildcard_ctx, .disk_required = 1, .need_peer = 1, ACF4_ADVANCED_NEED_VOL};
 static struct adm_cmd new_current_uuid_cmd = {"new-current-uuid", adm_drbdsetup, &new_current_uuid_cmd_ctx, ACF4_ADVANCED_NEED_VOL .backend_res_name = 0};
 static struct adm_cmd check_resize_cmd = {"check-resize", adm_chk_resize, ACF4_ADVANCED};
 
@@ -465,7 +465,6 @@ struct adm_cmd *cmds[] = {
 	&sh_md_idx_cmd,
 	&sh_ip_cmd,
 	&sh_lr_of_cmd,
-	&sh_status_cmd,
 
 	&proxy_up_cmd,
 	&proxy_down_cmd,
@@ -770,19 +769,23 @@ int run_deferred_cmds(void)
 
 static int adm_adjust(const struct cfg_ctx *ctx)
 {
-	int adjust_flags = ADJUST_DISK | ADJUST_NET;
+	static int adjust_flags = 0;
 	struct d_name *opt;
 
-	opt = find_backend_option("--skip-disk");
-	if (opt) {
-		STAILQ_REMOVE(&backend_options, opt, d_name, link);
-		adjust_flags &= ~ADJUST_DISK;
-	}
+	if (!adjust_flags) {
+		opt = find_backend_option("--skip-disk");
+		if (opt)
+			STAILQ_REMOVE(&backend_options, opt, d_name, link);
+		else
+			adjust_flags |= ADJUST_DISK;
 
-	opt = find_backend_option("--skip-net");
-	if (opt) {
-		STAILQ_REMOVE(&backend_options, opt, d_name, link);
-		adjust_flags &= ~ADJUST_NET;
+		opt = find_backend_option("--skip-net");
+		if (opt)
+			STAILQ_REMOVE(&backend_options, opt, d_name, link);
+		else
+			adjust_flags |= ADJUST_NET;
+
+		adjust_flags |= ADJUST_SKIP_CHECKED;
 	}
 
 	return _adm_adjust(ctx, adjust_flags);
@@ -790,6 +793,8 @@ static int adm_adjust(const struct cfg_ctx *ctx)
 
 static int sh_nop(const struct cfg_ctx *ctx)
 {
+	if (!config_valid)
+		return 10;
 	return 0;
 }
 
@@ -1026,7 +1031,7 @@ static void find_drbdcmd(char **cmd, char **pathes)
 static bool is_valid_backend_option(const char* name, const struct context_def *context_def)
 {
 	const struct field_def *field;
-	/* options have a leading "--", while field names do not have that -> name + 2 */
+	int len_to_equal_sign_or_nul;
 
 	if (context_def == &wildcard_ctx)
 		return true;
@@ -1034,8 +1039,13 @@ static bool is_valid_backend_option(const char* name, const struct context_def *
 	if (!context_def || strlen(name) <= 2)
 		return false;
 
+	/* options have a leading "--", while field names do not have that */
+	name += 2;
+	/* compare only until first equal sign, if any */
+	len_to_equal_sign_or_nul = strcspn(name, "=");
+
 	for (field = context_def->fields; field->name; field++) {
-		if (!strcmp(name + 2, field->name))
+		if (!strncmp(name, field->name, len_to_equal_sign_or_nul))
 			return true;
 	}
 	return false;
@@ -1098,6 +1108,8 @@ static int adm_attach(const struct cfg_ctx *ctx)
 	argv[NA(argc)] = (char *)ctx->cmd->name; /* "attach" : "disk-options"; */
 	argv[NA(argc)] = ssprintf("%d", vol->device_minor);
 	if (do_attach) {
+		assert(vol->disk != NULL);
+		assert(vol->disk[0] != '\0');
 		argv[NA(argc)] = vol->disk;
 		if (!strcmp(vol->meta_disk, "internal")) {
 			argv[NA(argc)] = vol->disk;
@@ -1219,6 +1231,8 @@ int adm_resize(const struct cfg_ctx *ctx)
 	struct d_option *opt;
 	bool is_resize = !strcmp(ctx->cmd->name, "resize");
 	off64_t old_size = -1;
+	off64_t target_size = 0;
+	off64_t new_size;
 	int argc = 0;
 	int silent;
 	int ex;
@@ -1229,8 +1243,13 @@ int adm_resize(const struct cfg_ctx *ctx)
 	opt = find_opt(&ctx->vol->disk_options, "size");
 	if (!opt)
 		opt = find_opt(&ctx->res->disk_options, "size");
-	if (opt)
+	if (opt) {
 		argv[NA(argc)] = ssprintf("--%s=%s", opt->name, opt->value);
+		target_size = m_strtoll(opt->value, 's');
+		/* FIXME: what if "add_setup_options" below overrides target_size
+		 * with an explicit, on-command-line target_size? */
+	}
+
 	add_setup_options(argv, &argc, ctx->cmd->drbdsetup_ctx);
 	argv[NA(argc)] = 0;
 
@@ -1241,6 +1260,15 @@ int adm_resize(const struct cfg_ctx *ctx)
 	silent = !strcmp(ctx->cmd->name, "check-resize") ? SUPRESS_STDERR : 0;
 	ex = m_system_ex(argv, SLEEPS_SHORT | silent, ctx->res->name);
 
+	if (ex && target_size) {
+		new_size = read_drbd_dev_size(ctx->vol->device_minor);
+		if (new_size == target_size) {
+			fprintf(stderr, "Current size of drbd%u equals target size (%llu byte), exit code %d ignored.\n",
+				ctx->vol->device_minor, (unsigned long long)new_size, ex);
+			ex = 0;
+		}
+	}
+
 	if (ex)
 		return ex;
 
@@ -1263,11 +1291,7 @@ int adm_resize(const struct cfg_ctx *ctx)
 	   changed, but only up to 10 seconds if know the target size, up to
 	   3 seconds waiting for some change. */
 	if (old_size > 0) {
-		off64_t target_size, new_size;
-		int timeo;
-
-		target_size = m_strtoll(get_opt_val(&ctx->vol->disk_options, "size", "0"), 's');
-		timeo = target_size ? 100 : 30;
+		int timeo = target_size ? 100 : 30;
 
 		do {
 			new_size = read_drbd_dev_size(ctx->vol->device_minor);
@@ -1299,6 +1323,8 @@ int _adm_drbdmeta(const struct cfg_ctx *ctx, int flags, char *argument)
 	argv[NA(argc)] = ssprintf("%d", vol->device_minor);
 	argv[NA(argc)] = "v09";
 	if (!strcmp(vol->meta_disk, "internal")) {
+		assert(vol->disk != NULL);
+		assert(vol->disk[0] != '\0');
 		argv[NA(argc)] = vol->disk;
 	} else {
 		argv[NA(argc)] = vol->meta_disk;
@@ -1418,67 +1444,6 @@ static int __adm_drbdsetup_silent(const struct cfg_ctx *ctx)
 	return rv;
 }
 
-int sh_status(const struct cfg_ctx *ctx)
-{
-	struct cfg_ctx tmp_ctx = *ctx;
-	struct d_resource *r;
-	struct d_volume *vol, *lower_vol;
-	int rv = 0;
-
-	if (!dry_run) {
-		printf("_drbd_version=%s\n_drbd_api=%u\n",
-		       shell_escape(PACKAGE_VERSION), API_VERSION);
-		printf("_config_file=%s\n\n\n", shell_escape(config_save));
-	}
-
-	for_each_resource(r, &config) {
-		if (r->ignore)
-			continue;
-		tmp_ctx.res = r;
-
-		printf("_conf_res_name=%s\n", shell_escape(r->name));
-		printf("_conf_file_line=%s:%u\n\n", shell_escape(r->config_file), r->start_line);
-		if (r->stacked && r->me->lower) {
-			printf("_stacked_on=%s\n", shell_escape(r->me->lower->name));
-			lower_vol = STAILQ_FIRST(&r->me->lower->me->volumes);
-		} else {
-			/* reset stuff */
-			printf("_stacked_on=\n");
-			printf("_stacked_on_device=\n");
-			printf("_stacked_on_minor=\n");
-			lower_vol = NULL;
-		}
-		/* TODO: remove this loop, have drbdsetup use dump
-		 * and optionally filter on resource name.
-		 * "stacked" information is not directly known to drbdsetup, though.
-		 */
-		for_each_volume(vol, &r->me->volumes) {
-			/* do not continue in this loop,
-			 * or lower_vol will get out of sync */
-			if (lower_vol) {
-				printf("_stacked_on_device=%s\n", shell_escape(lower_vol->device));
-				printf("_stacked_on_minor=%d\n", lower_vol->device_minor);
-			} else if (r->stacked && r->me->lower) {
-				/* ASSERT */
-				err("in %s: stacked volume[%u] without lower volume\n",
-				    r->name, vol->vnr);
-				abort();
-			}
-			printf("_conf_volume=%d\n", vol->vnr);
-
-			tmp_ctx.vol = vol;
-			rv = _adm_drbdsetup(&tmp_ctx, SLEEPS_SHORT);
-			if (rv)
-				return rv;
-
-			if (lower_vol)
-				lower_vol = STAILQ_NEXT(lower_vol, link);
-			/* vol is advanced by for_each_volume */
-		}
-	}
-	return 0;
-}
-
 static int adm_outdate(const struct cfg_ctx *ctx)
 {
 	int rv;
@@ -1599,7 +1564,7 @@ static int adm_khelper(const struct cfg_ctx *ctx)
 		snprintf(volume_string, sizeof(volume_string), "%u", vol->vnr);
 		setenv("DRBD_MINOR", minor_string, 1);
 		setenv("DRBD_VOLUME", volume_string, 1);
-		setenv("DRBD_LL_DISK", shell_escape(vol->disk), 1);
+		setenv("DRBD_LL_DISK", shell_escape(vol->disk ?: "none"), 1);
 	} else {
 		char *minor_list;
 		char *volume_list;
@@ -1614,7 +1579,7 @@ static int adm_khelper(const struct cfg_ctx *ctx)
 
 		for_each_volume(vol, &res->me->volumes) {
 			volumes++;
-			ll_len += strlen(shell_escape(vol->disk)) + 1;
+			ll_len += strlen(shell_escape(vol->disk ?: "none")) + 1;
 		}
 
 		/* max minor number is 2**20 - 1, which is 7 decimal digits.
@@ -1645,7 +1610,7 @@ static int adm_khelper(const struct cfg_ctx *ctx)
 
 			append(minor, "%d", vol->device_minor);
 			append(volume, "%d", vol->vnr);
-			append(ll, "%s", shell_escape(vol->disk));
+			append(ll, "%s", shell_escape(vol->disk ?: "none"));
 
 #undef append
 			separator = " ";
@@ -1672,7 +1637,7 @@ int adm_peer_device(const struct cfg_ctx *ctx)
 	char *argv[MAX_ARGS];
 	int argc = 0;
 
-	peer_device = find_peer_device(conn, vol->vnr);
+	peer_device = find_peer_device(res->me, conn, vol->vnr);
 	if (!peer_device) {
 		err("Could not find peer_device object!\n");
 		exit(E_THINKO);
@@ -1970,7 +1935,7 @@ static int adm_up(const struct cfg_ctx *ctx)
 		tmp_ctx.conn = conn;
 
 		schedule_deferred_cmd(&new_peer_cmd, &tmp_ctx, CFG_NET_PREP_UP);
-		schedule_deferred_cmd(&new_path_cmd, &tmp_ctx, CFG_NET_PREP_UP);
+		schedule_deferred_cmd(&new_path_cmd, &tmp_ctx, CFG_NET_PATH);
 		schedule_deferred_cmd(&connect_cmd, &tmp_ctx, CFG_NET_CONNECT);
 
 		STAILQ_FOREACH(peer_device, &conn->peer_devices, connection_link) {
@@ -2928,6 +2893,10 @@ int parse_options(int argc, char **argv, struct adm_cmd **cmd, char ***resource_
 		case 't':
 			config_test = optarg;
 			break;
+		case 'E':
+			/* Remember as absolute name */
+			was_file_already_seen(optarg);
+			break;
 		case 's':
 			{
 				char *pathes[2];
@@ -3042,7 +3011,8 @@ int parse_options(int argc, char **argv, struct adm_cmd **cmd, char ***resource_
 				break;
 
 		field = NULL;
-		if (option[0] == '-' && option[1] == '-' && (*cmd)->drbdsetup_ctx) {
+		if (option[0] == '-' && option[1] == '-' && (*cmd)->drbdsetup_ctx &&
+		    (*cmd)->drbdsetup_ctx != &wildcard_ctx) {
 			for (field = (*cmd)->drbdsetup_ctx->fields; field->name; field++) {
 				if (strlen(field->name) == len - 2 &&
 				    !strncmp(option + 2, field->name, len - 2))
@@ -3051,7 +3021,7 @@ int parse_options(int argc, char **argv, struct adm_cmd **cmd, char ***resource_
 			if (!field->name)
 				field = NULL;
 		}
-		if (!field) {
+		if (!field && (*cmd)->drbdsetup_ctx != &wildcard_ctx) {
 			err("%s: unrecognized option '%.*s'\n", progname, len, option);
 			goto help;
 		}
